//****************************************************************************
// @Module        UART (Serial Interface)
// @Filename      UART.C
// @Project       FlashExampleXC88x.dav
//----------------------------------------------------------------------------
// @Controller    Infineon XC886CLM-8FF
//
// @Compiler      Keil
//
// @Codegenerator 1.9
//
// @Description:  This file contains functions that use the UART module.
//
//----------------------------------------------------------------------------
// @Date          13.05.2011 17:15:13
//
//****************************************************************************

// USER CODE BEGIN (UART_General,1)

// USER CODE END



//****************************************************************************
// @Project Includes
//****************************************************************************

#include "MAIN.H"

// USER CODE BEGIN (UART_General,2)
#include "FlashHandler.h"
// USER CODE END


//****************************************************************************
// @Macros
//****************************************************************************

// USER CODE BEGIN (UART_General,3)

// USER CODE END


//****************************************************************************
// @Defines
//****************************************************************************

// USER CODE BEGIN (UART_General,4)

// USER CODE END


//****************************************************************************
// @Typedefs
//****************************************************************************

// USER CODE BEGIN (UART_General,5)

// USER CODE END


//****************************************************************************
// @Imported Global Variables
//****************************************************************************

// USER CODE BEGIN (UART_General,6)
extern unsigned  char RecCount;
extern unsigned char xdata RecBuffer[BYTES_PER_WORDLINE];
extern unsigned char xdata TrxBuffer[BYTES_PER_WORDLINE];
extern unsigned char  RecBufFull;
extern volatile unsigned char gState;
// USER CODE END


//****************************************************************************
// @Global Variables
//****************************************************************************

// USER CODE BEGIN (UART_General,7)
unsigned char RxdBuf;
static unsigned char value;
// USER CODE END


//****************************************************************************
// @External Prototypes
//****************************************************************************

// USER CODE BEGIN (UART_General,8)

// USER CODE END


//****************************************************************************
// @Prototypes Of Local Functions
//****************************************************************************

// USER CODE BEGIN (UART_General,9)
void UART_RXD1(void);
// USER CODE END


//****************************************************************************
// @Function      void UART_vInit(void) 
//
//----------------------------------------------------------------------------
// @Description   This is the initialization function of the UART function 
//                library. It is assumed that the SFRs used by this library 
//                are in their reset state. 
//                
//                The following SFR fields will be initialized:
//                - register SCON
//                - bits SMOD and ES
//
//----------------------------------------------------------------------------
// @Returnvalue   None
//
//----------------------------------------------------------------------------
// @Parameters    None
//
//----------------------------------------------------------------------------
// @Date          13.05.2011
//
//****************************************************************************

// USER CODE BEGIN (UART_Init,1)

// USER CODE END

void UART_vInit(void)
{
  // USER CODE BEGIN (UART_Init,2)

  // USER CODE END

  ///  -----------------------------------------------------------------------
  ///  UART settings
  ///  -----------------------------------------------------------------------
  ///  Pin TXD_0 (P1.1) is selected for transmission
  ///  Pin RXD_0 (P1.0) is selected for reception
  ///  Receiver enabled
  ///  Mode 1: 8-bit data, 1 start bit, 1 stop bit, variable baud rate
  ///  Receiver interrupt flag RI will only be activated if a valid stop bit 
  ///  was received
  ///  BRG is selected for baudrate generation

  SFR_PAGE(_pp2, noSST);         // switch to page 2 without saving
  P1_ALTSEL0   &= ~(ubyte)0x02;  // configure alternate function register 0
  P1_ALTSEL1   |=  (ubyte)0x02;  // configure alternate function register 1
  SFR_PAGE(_pp0, noSST);         // switch to page 0 without saving
  P1_DIR       |=  (ubyte)0x02;  // set output direction


  MODPISEL     &= ~(ubyte)0x01;  // configure peripheral input select register
  BCON          =  0x00;         // reset baudrate timer/reload register
  SCON          =  0x70;         // load serial channel control register

  ///  -----------------------------------------------------------------------
  ///  Baudrate generator settings
  ///  -----------------------------------------------------------------------
  ///  input clock = fPCLK
  ///  Fractional divider is disabled
  ///  baudrate = 9,6154 kbaud

  BG            =  0x9B;         // load baudrate timer/reload register
  BCON         |=  0x01;         // load baud rate control register


  // USER CODE BEGIN (UART_Init,3)

  // USER CODE END

  ///  UART interrupt enabled
  ES = 1;

} //  End of function UART_vInit


//****************************************************************************
// @Function      void UART_viIsr(void) 
//
//----------------------------------------------------------------------------
// @Description   This is the service routine for the UART interrupt. It is 
//                called after each transmission (flag TI set) or reception 
//                (flag RI set) of a data unit.
//                Please note that you have to add application specific code 
//                to this function.
//
//----------------------------------------------------------------------------
// @Returnvalue   None
//
//----------------------------------------------------------------------------
// @Parameters    None
//
//----------------------------------------------------------------------------
// @Date          13.05.2011
//
//****************************************************************************

//   You have two choices for interrupt type select in Project Settings Page 
//   under Global Settings Section.
//   If you select CHOICE 1 then ISR will be generated with push and pop.
//   If you select CHOICE 2 then ISR will be generated without push and pop.
//   Default choice is CHOICE 2.
//   Current selection is CHOICE 2 

// USER CODE BEGIN (UART_Isr,1)

// USER CODE END

void UART_viIsr(void) interrupt UARTINT
{

    // USER CODE BEGIN (UART_Isr,2)

    // USER CODE END
  SFR_PAGE(_su0, SST0);          // switch to page 0
  if (TI)
  {
    // USER CODE BEGIN (UART_Isr,3)

       //TI = 0;

    // USER CODE END
  }
  if (RI)
  {
    // USER CODE BEGIN (UART_Isr,4)
       RI = 0;
	   UART_RXD1();

    // USER CODE END
  }

    // USER CODE BEGIN (UART_Isr,5)

    // USER CODE END
  SFR_PAGE(_su0, RST0);          // restore the old page
} //  End of function UART_viIsr


// USER CODE BEGIN (UART_General,10)
void UART_RXD1(void)
{
// receives BYTES_PER_WORDLINE bytes and writes them into buffer, then transmits first byte out of buffer
	value = SBUF;
	switch (gState)
	{
	  case 0:
		  if (value == 0x31) 
			{ gState = 1; }					// Choose Menu 1 
		  else 
			if (value == 0x32) 
				{ gState = 2; }				// Choose Menu 2
		    else 
				if (value == 0x33) 
					{ gState = 3; }			// Choose Menu 3
				else 
					if (value == 0x34) 
						{ gState = 4; }		// Choose Menu 4
		    
		 
		SBUF = value;
		while(TI==0) {;}
		TI = 0;
		RecCount++;
	    break;

	  case 2:
		   
	    if (value == 0xD) {  // Carriage Return
			 
	       while (RecCount < BYTES_PER_WORDLINE) {RecBuffer[RecCount++] = 0;}
		   RecCount = 0;
		   RecBufFull = 1;
	    } else if (RecCount == (BYTES_PER_WORDLINE - 1) && value != 0x8) {
		   // Receive Buffer is full
			 
	       SBUF = 0x7;       // Bell
	    } else if (value == 0x8 && RecCount > 0) {
		   // Backspace Character
	       SBUF = 0x8;
		   while(TI==0) {;}
		   TI = 0;
		   SBUF = 0x0;
		   while (TI==0) {;}
		   TI = 0;
		   SBUF = value;
		   RecCount--;
	    } else if (value != 0x8) {
		   // Normal Character
			
	       if (!RecBufFull)	RecBuffer[RecCount++] = value;
		   SBUF = value;
		   
	    }
		break;
	  
	  default:
	    break;
	}
}
// USER CODE END

